/*
	NEXUS2tgf.cc, part of
	
	treegraph
	
	Tree formatting program
	
	Generates vector graphics (SVG,EPS) from .tgf-tree description files.

	Copyright (c) 2003-06 by Joern Mueller


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program (GPL.html); if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	(also: http://www.gnu.org)
*/

#include "tgf-main.h"

#include <iostream>
#include <cctype>
#include <cstdio>
#include <cstdlib>

#include "NEXUS2tgf.h"
#include "tgf-error.h"

using namespace std;

const char kILs[]="()[]{}/\\,;:=*'\"`+-<>";
const int kNEXUSIdentifier=0;


inline bool isIdentifierLetter(char ch){
	if(ch<=' ') return false;
	for(int i=0;kILs[i];i++){
		if(ch==kILs[i]) return false;
	}
	return true;
}

NEXUS_tree_import::NEXUS_tree_import(std::ifstream& eing, std::ofstream& ausg)
: parser(eing), aus(ausg)
{
	altNexus=true;
	brlens=false;
	intLabels=false;
	
	translateToTGF();

}

int NEXUS_tree_import::translateToTGF()
//return: 0 wenn alles in Ordnung
{
	std::string token="";
	getnexttoken(token);
	if(!lcEqual(token,"#nexus")){
		fehler("Missing #NEXUS");
	}
//Bloecke lesen
	
	int ntaxa=0;
	bool done=false;
	while(!done){
		getnexttoken(token);
		if(!lcEqual(token,"begin")){
			fehler("Missing BEGIN statement");
		}
		getnexttoken(token);//name of NEXUS block
		if(lcEqual(token,"trees")){
			
			ignoreSymbol(';');
			
			int numoftrees=0;
			do {
				getnexttoken(token);
				if(lcEqual(token,"translate")){
					int tv;
					altNexus=false;
					do{
						for(int j=0;j<2;j++){
							tv=getnexttoken(token);//number
							if(tv!=kNEXUSIdentifier){	
								fehler("Expected identifier");
							}
						}	
						taxa.push_back(token);
						tv=getnexttoken(token);// Komma oder ;
						if(tv!=';' && tv!=','){
							fehler("Expected identifier");
						}
					}while(good() && !eof() && tv!=';');
				} else if(lcEqual(token,"tree")){
					getnexttoken(token); //name of tree or * before it
					if(token=="*") getnexttoken(token);
					if(++numoftrees>1){
						cout<<"Tree "+token+" in TREES-block will be ignored.";
						token="end";
					} else {
						cout << "Found tree "<< token<< endl;
						while(lies()!='='){
							if(!good() || eof()){
								fehler("Missing '=' after tree name");
							}
						}
						double bl=static_cast<double>(12);
						if (!altNexus) {
								bl=(2.8346457*250)/(double)taxa.size();
								if (bl>18) bl=18;
						} 	
						writesettings(bl,token);

						ntaxa=readTreeLine ();
						
					}
				} else if(lcEqual(token,"end") || lcEqual(token,"endblock")){
					
					done=true;
				} else {
					cout<<"Ignoring line " << token<<endl;
					ignoreline(token);
				}
			} while(!done && good() && !eof());
			
			
		} else {
			cout<<"Ignoring block " << token<<endl;
			ignoreblock(token);
		}
		
	}

	cout << "Import from ";
	cout << (altNexus?"altNEXUS":"NEXUS");
	cout << " file with";
	if (!brlens) cout << "out";
	cout << " branchlengths and with";
	if (!intLabels) cout << "out";
	cout << "\n internal node labels.\n";
	cout << ntaxa << " taxa read.\n";
	
	
	
	return ntaxa>0?0:1;
}

int NEXUS_tree_import::readTreeLine ()
// return: Numer of taxa read. 0 when error occured
{
	std::string token="";

	//beginnt bei (
	int ch,last= -1;
	int ntaxa=taxa.size();
	int blevel=0;
	float f;


	while((ch=getnexttoken(token))!=EOF){
		switch(ch){
		case '(':
			aus << '(';
			++blevel;
			break;
		case ',':
			aus << ',';
			break;
		case ')':
			aus << ')';
			if(--blevel<0){
				fehler("Too many ')' or missing '('");
			}
			break;
		case ':': //read branch length
			brlens=true;
			parser::ein >> f;
			aus << "\\len{" << f << "}";
			break;
		case ';':
			if(blevel>0){
				fehler("Missing ')' or too many '('\n");
			} else
				return ntaxa;
		case kNEXUSIdentifier:
			aus << '\"';
			if(last!=')'){
				//taxon, not label
				if (taxa.empty()) {
					aus << token;
					++ntaxa;
				} else {
					int txnr=atoi(token.c_str())-1;
					if(txnr<0 || txnr>=static_cast<int>(taxa.size())){
						fehler("translate " +token+ ": Out of range.");
					}
					aus << taxa[txnr];
				}
			} else {
				//label
				aus << token;
			}
			aus << "\"\n";
			break;
		default:
			fehler("Illegal token "+tgfError::CharToString(ch));
		}//switch
		last=ch;
	}
	fehler("unexpected end of tree");
	
	return 0;
}



int NEXUS_tree_import::getnexttoken(std::string& token)
{
	skipspaces();

	int ch;
	while((ch=lies())=='['){
		skipcomment();
		skipspaces();
	}
	
	token="";
	int typ;
	if(ch=='\''){
		while((ch=lies())!=EOF && ch!='\''){
			if(ch=='\n' || ch=='\r'){
				fehler("line break in identifier.");
			}
			token+=ch;
		}
		typ=kNEXUSIdentifier;
	} else if(isIdentifierLetter(ch)){
		do{
			if(ch=='_') ch=' ';
			token+=ch;
		}while((ch=lies())!=EOF && isIdentifierLetter(ch));
		zurueck();
		typ=kNEXUSIdentifier;
	} else {
		token+=ch;
		typ=ch;
	}
	return typ;
}

void NEXUS_tree_import::ignoreSymbol(int sym)
{
	skipspaces();
	if(lies()!=sym){
		char s[10]="Missing  ";
		s[8]=static_cast<char>(sym);
		s[9]=0;
		fehler(s);
	}


}


void NEXUS_tree_import::ignoreblock(const std::string& blockname)
{
	std::string token="";
	
	while(good() && !eof()){
		getnexttoken(token);
		if(lcEqual(token,"end") || lcEqual(token,"endblock")){
			return ignoreSymbol(';');
		} else ignoreline(token);
		
	}

}

void NEXUS_tree_import::ignoreline(const std::string& linename)
{
	int ch;
	while((ch=lies())!=';' && ch!=EOF){
		if(ch=='[')
			skipcomment();
	}
	if(ch!=';'){
		fehler("Missing ';' in NEXUS line " + linename);
	}
	
}


void NEXUS_tree_import::skipcomment()
{
	int b=1;
	while(b>0){
		int ch=lies();
		if(ch==EOF){
			fehler("Missing end of comment");
		}
		if(ch==']') --b;
		else if(ch=='[') ++b; 
	}
	
}


void NEXUS_tree_import::writesettings (double schrgr, const string& treename)
{
	int defaultWidth=150, defaultHeight=250;
	aus << "%\n%\n% NOTE: This tree has been imported from NEXUS.\n";
	aus << "%       You should use 'save' (command s) to make the\n";
	aus << "%       tree part more readable.\n%\n\n";
	aus << "\\begindef\n";
	if(treename.size()>0){
		aus << "\n    \\name{"<<treename<<"} % imported from NEXUS file\n";
	}
	aus << "    \\version{" TGFFILEFORMAT "} % version of treegraph file format\n\n";
	aus << "%\n% tree geometry\n%\n";
	aus << "    \\width{" << defaultWidth << "}  % width in mm\n";
	aus << "    \\height{" << defaultHeight << "}  % height in mm\n";
	aus << "%     \\autolength  % ignore len{} in nodes\n";
	aus << "%     \\variable\n\n";
	aus << "%\n% layout\n%\n";
	aus << "    \\roundness{0.2}  % between 0.0 and 1.0\n";
	aus << "    \\thickness{0.3}  % in mm\n";
	aus << "    \\margin{0}{0}{0}{0} % left top right bottom, in mm\n";
	aus << "    \\style{default}{plain}{"<<schrgr*0.8<<"}  \n";
	aus << "    \\style{r}{plain}{"<<schrgr<<"}  \n";
	aus << "\n%     \\style{u1}{plain}{"<<schrgr*0.8<<"}\n";
	aus << "%     \\style{d1}{italic}{"<<schrgr*0.8<<"}\n";
	aus << "%     \\style{u2}{bold}{"<<schrgr*0.7<<"}\n";
	aus << "%     \\style{br}{plain}{"<<schrgr*1.2<<"}\n";
	aus << "%     \\rulepos{5}{20}\n % position of rule\n";
	aus << "%     \\separator{:}\n";
	aus << "%     \\proof\n\n";

	aus << "%\n% brackets\n%\n";
	aus << "%     \\bracket{0}{n1}{n2}{example text}\n";
	aus << "%     \\brace*{1.5}{root}{root}{example text}\n\n";
	aus << "\\enddef\n\n";
	aus.flush();
}






