/*
	tgf-edit.cc, part of
	
	treegraph
	
	Tree formatting program
	
	Generates vector graphics (SVG,EPS) from .tgf-tree description files.

	Copyright (c) 2003-06 by Joern Mueller


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program (GPL.html); if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	(also: http://www.gnu.org)
*/







#include "tgf-tree.h"
#include "tgf-error.h"
#include <cstdlib>

void tree::edit()
{
using namespace std;

	cout << "Type '?' for help.\n";
	while(1){
		cout << "\n> ";
	
		string txt;
		getline(cin,txt);
		if(txt[0]=='x') break;
		else if(txt[0]=='?'){
			cout << "Available commands: (<L> is node label; label of whole tree is 'root')\n\n";
			cout << "swap <L>     swaps sublineages of <L>. Same as perm <L> 2 1\n";
			cout << "perm <L> i_1 i_2 .. i_n  permutes the n sublineages of <L>\n";
			cout << "dump <L>     writes subtree (the clade characterized by <L>) to file\n";
			cout << "ladder <L>   ladderizes right\n";
			cout << "dladder <L>  ladderizes left\n";
			cout << "mladder <L>  symmetric ladderization\n";
			cout << "coll <L>     collapses <L>\n";
			cout << "mv <L> <L2>  makes <L> a child node of <L2>\n";
			cout << "del <L>      deletes all sublineages of <L>, leaving <L> as terminal\n";
			cout << "root <L>     inserts new root between <L> and its ancestral node\n";
			cout << "x            quits edit mode\n";
			cout << endl;
			continue;
		}
		
		vector<string> arglist;
		arglist.reserve(5);
		for(;;) {
			string::size_type p=txt.find(' ');
			arglist.push_back(txt.substr(0,p));
			if(p!=string::npos) txt=txt.substr(p+1);
			else break;
		}
		
		//alle Befehle ausser 'x' haben ein Argument
		string cmd=arglist[0];
		if(arglist.size()<2){
			cout << "Unknown: '" << cmd << "'\n";
			cout << "Type '?' for help.\n";
			continue;
		}
		const tnode *nd=lb.bfind_label(arglist[1].c_str());
		if(nd==0){
			cout <<  "Node with label '" << arglist[1] << "' does not exist.\n";
			continue;
		}
		if(cmd=="ladder" || cmd=="dladder"){
			((tnode*)nd)->ladderize_sichtbare((cmd=="ladder")?1:-1);
			drty=true;
		} else if(cmd=="mladder"){
			((tnode*)nd)->symladderize(1);
			drty=true;	
		} else if(cmd=="swap"){
			((tnode*)nd)->swap_children();
			drty=true;
		} else if(cmd=="perm"){
			list<int> pm;
			for(unsigned int i=2; i<arglist.size(); i++){
				int k=atoi(arglist[i].c_str());
				pm.push_back(k);
			}
			if(((tnode*)nd)->permute(pm))
				drty=true;
			else cerr << "permute failed.\n";
		} else if(cmd=="dump"){
			if(arglist[1]=="root") nd=0;
			cout << "Save as: ";
			string fname;
			cin >> fname;
			ofstream ausg(fname.c_str());
			cout << "Saving...\n";
			write(ausg,nd);
			cout << "Done.\n";
		} else if(cmd=="del"){
			remove_subtree((tnode*)nd);
			drty = true;
		} else if(cmd=="mv"){
			cout << "Move " << arglist[1] << " to " << arglist[2] << endl;
			const tnode *nd2=lb.bfind_label(arglist[2].c_str());
			if(nd2==0){
				cerr <<  "Node with label '" << arglist[2] << "' does not exist.\n";
				continue;
			}
			if(nd->prev==0){
				cerr << "root can not be moved. Use \"root\" command first.\n";
				continue;
			}
			if(nd->prev==nd2){
				cerr << arglist[1] << " is already a child node of " << arglist[2] << endl;
				continue;
			}
			(((tnode*)nd)->prev)->removechild((tnode*)nd);
			((tnode*)nd2)->insertchild((tnode*)nd);
			drty=true;
		} else if(cmd=="coll"){
			if(nd->has_label())
				lb.kill_label(nd->label());
			((tnode*)nd)->collapse();
			drty=true;
		} else if(cmd=="root"){
			try {
				if(reroot((tnode*)nd)){
					drty=true;
				}
			}
			catch(tgfError::error e){
				cerr << e.message();
			}
		} else {
			cout << "Unknown: '" << cmd << "'\n";
			cout << "Type '?' for help.\n";
		}		
	}
	
	if(dirty()){
		cout << "Recalculating node positions...\n";
		calcpos();
	}
}
