/*
	tgf-gostream.h, part of
	
	treegraph
	
	Tree formatting program
	
	Generates vector graphics (SVG,EPS) from .tgf-tree description files.

	Copyright (c) 2003-06 by Joern Mueller


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program (GPL.html); if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	(also: http://www.gnu.org)
*/







#ifndef __TGF_GOSTREAM_H__
#define __TGF_GOSTREAM_H__

#include <string>
#include <iostream>
#include "tgf-utils.h"
#include "tgf-treedesc.h"

class gostream
{
protected:
	std::ostream& s;
	std::string fontpath;
public:
	gostream(std::ostream &s1);

	virtual void moveto(float x,float y)=0;
	virtual void lineto(float x,float y)=0;
	virtual void curveto(float,float, float,float, float,float)=0;

	virtual void write_head(const fRect& paper, const fRect& bbox,float)=0;
	virtual void write_foot()=0;
	virtual void write_text(const nfstring& s, const fPoint& p, float rotate=0)=0;
	virtual void write_rgabel(float,float,float,float,float,float)=0;
	virtual void write_egabel(float,float,float,float,float)=0;
	virtual void write_hline(float,float,float)=0;
	virtual void write_klammer(const nfstring&,float,float,float,float,int)=0;
	virtual void write_rule(const fPoint& pos, float z,float wd)=0;
	virtual void write_arrow(const fPoint& pos, float angle)=0;	
	virtual void write_nodelabel(const fPoint& pos, const std::string& s)=0;
	
	void set_fontpath(std::string&);

};


class svg_ostream : public gostream
{
	tree_desc dsc1;
public:
	svg_ostream(std::ostream& s1,const tree_desc& dsc) ;

	void moveto(float x,float y);
	void lineto(float x,float y);
	void curveto(float,float, float,float, float,float);

	void write_head(const fRect& paper, const fRect& bbox,float);
	void write_foot();
	void write_text(const nfstring& s, const fPoint& p, float rotate=0);
	void write_rgabel(float,float,float,float,float,float);
	void write_egabel(float,float,float,float,float);
	void write_hline(float,float,float);
	void write_klammer(const nfstring&,float,float,float,float,int);
	void write_rule(const fPoint& pos, float z,float wd);
	void write_arrow(const fPoint& pos, float angle);

	void write_nodelabel(const fPoint& pos, const std::string& s);
};


class ps_ostream : public gostream
{
	float yh;
	inline float ty(float y) const { return yh-y;}
	fontstyle laststyle;
	bool finited;
	tree_desc dsc1;

	void setfont(const fontstyle& f);
public:
	ps_ostream(std::ostream& s1, const tree_desc& dsc);

	void moveto(float x,float y);
	void lineto(float x,float y);
	void curveto(float,float, float,float, float,float);


	void write_head(const fRect& paper, const fRect& bbox,float);
	void write_foot();
	void write_text(const nfstring& s, const fPoint& p, float rotate=0);
	void write_rgabel(float,float,float,float,float,float);
	void write_egabel(float,float,float,float,float);
	void write_hline(float,float,float);
	void write_klammer(const nfstring&,float,float,float,float,int);
	void write_rule(const fPoint& pos, float z,float wd);
	void write_arrow(const fPoint& pos, float angle);

	void write_nodelabel(const fPoint& pos, const std::string& s);
};


#endif /* __TGF_GOSTREAM_H__ */


