/*
	tgf-main.cc, part of
	
	treegraph
	
	Tree formatting program
	
	Generates vector graphics (SVG,EPS) from .tgf-tree description files.

	Copyright (c) 2003-06 by Joern Mueller


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program (GPL.html); if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	(also: http://www.gnu.org)
*/



#include "tgf-main.h"
#include "tgf-tree.h"
#include "NEXUS2tgf.h"
#include "tgf-render.h"
#include "tgf-error.h"
#include <cstdlib>
#include <cstring>

#define LEERZEILE "\r                                                   \n"

#define WINDOWSDEVCPP 0 //set to 0

void getstring_d(std::string& a, const std::string& info, const std::string& def="");
void splitname(const std::string& s, std::string& r, std::string& t);
void showcommands();

int main(int nargs, char *argv[])
{
	using namespace std;
	

    string theFontPath="fonts.txt";
	bool interactive=false;
#if WINDOWSDEVCPP
	char *heimat=getenv("HOMEPATH");
    if(heimat){
		theFontPath=string(heimat)+"\\treegraph\\fonts.txt";
    }
#else
    char *heimat=getenv("HOME");
    if(heimat)
		theFontPath=string(heimat)+"/.treegraph/fonts.txt";
#endif

	string nam;
	tree *baum=0;
	char ch='p';//standard: als EPS schreiben
	
	if(nargs<=1){
		interactive=true;
		cout << "TreeGraph " TGFVERSION ", Copyright (C) 2003-06 by Joern Mueller, Kai Mueller.\n";
		cout <<"TreeGraph comes with NO WARRANTY; see GPL.html for details.\n\n";
		
	} else {
		for(int i=1; i<nargs; i++){
			if(argv[i][0]=='-'){
				if(strchr("pvest",argv[i][1]))
					ch=argv[i][1];
			} else {
				nam=argv[i];
				break;
			}
		}
	}

	
	string namroot="",namtail;


	baum=0;
	

	
	if(interactive)
		cout << "enter command or '?'\n";
	else {
		if(ch!='t'){
	//nur bei translate wird im batch-modus kein Baum benoetigt.
			ifstream eing2(nam.c_str());
			if(!eing2){
				cerr << "Error: File not found.\n";
				return 1;
			} else {
				if(baum) delete baum;
				baum =new tree();
				try {
					baum->read(eing2);
					splitname(nam,namroot,namtail);
				}
				catch(tgfError::syntax se){
					cerr << se.message()<<endl;
					delete baum;
					return 1;
				}
				catch(tgfError::error err){
					cerr << err.message()<<endl;
					delete baum;
					return 2;
				}
			}
		}
	}	

	do {
		if(interactive){
			if(!baum){
				cout << "\n>";
			} else {
				cout << endl << namroot << (baum->dirty()?"[*]> ":"> ");
			}
			flush(cout);
			string str1="?";
			getline(cin,str1,'\n');
			ch=str1[0];
			if(!baum){
				while(strchr("pveFsn",ch)){
					cerr<<"A tree must be loaded first (command 'l')\n";
					cout << "\n>";
					getline(cin,str1,'\n');
					ch=str1[0];
				}
		
			}

		}

		if(ch=='p' || ch=='v'){
			string tos=namroot+((ch=='v')?".svg":".eps");
			if(interactive){
				if(ch=='v'){
					getstring_d(tos,"SVG file name",tos);
				} else {
					getstring_d(tos,"EPS file name",tos);
				}
			}
			ofstream ausg(tos.c_str());
			if(!ausg){
				cerr << "Error: Cannot write to file "<<tos <<endl;
				if(!interactive) return 2;
			} else {
				cout << "writing to '"<<tos<< "'\n";
				try {
					if(ch=='v'){
						svg_ostream svgaus(ausg,baum->get_desc());
						baum->render(&svgaus);
					} else {
						ps_ostream psaus(ausg,baum->get_desc());
						psaus.set_fontpath(theFontPath);
						baum->render(&psaus);
					}
				}
				catch(tgfError::error ee){
					cerr << ee.message();
				}
			}
		} else if(ch=='n' || ch=='s'){
			string tos;
			getstring_d(tos,"Save as ",namroot+(ch=='n'?".nex":".tgf"));
			ofstream ausg(tos.c_str());
			if(!ausg){
				cerr << "Error: Cannot write to file "<<tos <<endl;
				if(!interactive) return 2;
			} else {
				cout << "Saving as '" << tos << "' ...";
				flush(cout);
				
				ch=='n'?baum->writeNEXUS(ausg):baum->write(ausg);
				
				cout << LEERZEILE;
			}
		} else if(ch=='l'){
			if(namroot.length()>0){
				getstring_d(nam,"TGF-file",namroot+".tgf");
			} else {
				cout << "TGF-file: ";
				getline(cin,nam,'\n');
			}
			ifstream eing2(nam.c_str());
			if(!eing2){
				cerr << "Error: Cannot open file '"<<nam<<"'.\n";
				if(!interactive) return 1;
			} else {
				if(baum) delete baum;
				baum =new tree();
				try {
					baum->read(eing2);
					splitname(nam,namroot,namtail);
				}
				catch(tgfError::syntax se){
					cerr << se.message();
					delete baum;
					baum=0;
				}

			}
		} else if(ch=='t') {
    			string nam2,root2;
				if(interactive){
					cout << "Name of file to translate: ";
 					flush(cout);
					getline(cin,nam2,'\n');
				} else nam2=nam;  		
    			ifstream eing(nam2.c_str());
    			if(!eing){
	        		cerr << "Error: File '" << nam << "' not found.\n";
	        		if(!interactive) return 1;
        		} else {
					splitname(nam2,root2,namtail);
					nam2=root2+".tgf";
					if(namroot.length()==0) namroot=root2;
					if(interactive){
						ifstream tmp(nam2.c_str());
						if(tmp){
							string res;
							tmp.close();
							getstring_d(res,"File "+nam2+" already exists. Overwrite? (Y/n) ","n");
							if(res[0]!='Y')
								return 1;
					
						}
					}
					ofstream aus(nam2.c_str());
					try {
						NEXUS_tree_import(eing,aus);
					}
					catch(tgfError::syntax ex){
						cerr << ex.message();
					}
					eing.close();
					aus.close();
					cout << "\nTGF file written to "<< nam2 <<endl;
    			}
		} else if(ch=='e'){
			cout << "*************\n";
			cout << "* Edit mode *\n";
			cout << "*************\n";
			interactive=true; //Editieren ist interaktiv
			baum->edit();
		} else if(ch=='F'){
			baum->show_dsc();
		} else if(ch=='f'){
			cout << "The font path:\n";
			cout << theFontPath<<endl;
		} else if(ch=='q'){
			interactive=false;
		} else {
			showcommands();
		}
	} while(interactive);
	
#if WINDOWSDEVCPP
    system("PAUSE");
#endif

	return 0;
}


void getstring_d(std::string& a, const std::string& info, const std::string& def)
{
		std::cout << info;
		if(def.size()>0){
			 std::cout << " ["<<def<<"]";
		}
		std::cout << ": ";
		flush(std::cout);
		std::string st;
		std::getline(std::cin,st,'\n');
		a=(st.size()==0)?def:st;
}


void splitname(const std::string& s, std::string& r, std::string& t)
{
	std::string::size_type p=s.rfind('.');
	r=s.substr(0,p);
	t=(p!=std::string::npos)?s.substr(p):"";
}

void showcommands()
{
	std::cout << "available commands:\n\n";
	std::cout << "l  loads tree from TGF file\n";
	std::cout << "t  translates NEXUS to TGF file\n";
	std::cout << "s  saves current tree as TGF file\n";
	std::cout << "n  saves current tree as NEXUS file\n"; 
	std::cout << "p  writes current tree into EPS file\n";		
	std::cout << "v  writes current tree into SVG file\n";
	std::cout << "e  edit mode\n";
	std::cout << "F  show settings\n";
	std::cout << "f  show font path\n";
	std::cout << "q  quits program\n";
}

