/*
	tgf-parser.h, part of
	
	treegraph
	
	Tree formatting program
	
	Generates vector graphics (SVG,EPS) from .tgf-tree description files.

	Copyright (c) 2003-06 by Joern Mueller


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program (GPL.html); if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	(also: http://www.gnu.org)
*/


#ifndef __TGF_PARSER_H__
#define __TGF_PARSER_H__

#include <string>
#include <iostream>
#include <fstream>
#include <vector>

class parser {
public:
	parser(std::ifstream& s1); 
	
	int lies();
	void zurueck();

	void skipspaces();
	int curline() const {return line;}

	bool lcEqual(const std::string& a, const std::string& bLower);
	
	void fehler(const std::string&);
	
	bool eof() const;
	bool good() const; 

protected:
	std::ifstream& ein;

private:
	int line;
	int lastRead;


};

class tgfParser : public parser
{
public:
	tgfParser(std::ifstream& s1);

	enum ttoken
	{
		kInit=0,
	//Tree-part
		kOpenTree,
		kCloseTree,
		kNextTree,
		kText,	// "...Text.." wird zu U1 oder R
		kCommand, //any other command, e.g \r{}, \label
				//in definition-part also brace/bracket etc
	//definition-part
		kBeginDef,
		kEndDef,
		kComment,
		kNewline,
	//other
		kEOF,
		kErr
	};

	
	float readbracefloat();
//	float readbracelength();//in pt
	void readbracestring(std::string&);
	void readcommand(std::string& cmd);
	void skipcomment();
	bool skipbracestring();

	ttoken readnextword(std::string&, bool defpart=false);


};



const float millimeter=2.8346457;
const float centimeter=10*millimeter;
const float inch=72.0;

#endif /* __TGF_PARSER_H__ */
