/*
	tgf-tree.h, part of
	
	treegraph
	
	Tree formatting program
	
	Generates vector graphics (SVG,EPS) from .tgf-tree description files.

	Copyright (c) 2003-06 by Joern Mueller


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program (GPL.html); if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	(also: http://www.gnu.org)
*/








#ifndef __TGF_TREE_H__
#define __TGF_TREE_H__

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <list>

#include <cmath>
#include <cctype>
#include <cstdio>

#include "tgf-utils.h"
#include "tgf-treedesc.h"
#include "tgf-parser.h"
#include "tgf-gostream.h"


class tnode
{
public:
	char *lbl;
	nfstring r;
	nfstring u[4];
	nfstring d[4];
	fPoint pos;
	float len;// \len{...} (oder autolength)
	float minlen;//erst: 0, dann: (wenn autolength: Textbreite, dann ggf. \minlen)
	float minabove;
	float minbelow;
	bool userset[3]; //minlen,minabove,minbelow wurden explizit angegeben
	
	int depth;
	bool prune;

	tnode *prev;
	tnode *next;
	tnode *down;
	

	tnode(tnode *prv,  float l=autolength);

	void selection_sort_children(int);
	void swap_children();
	bool permute(std::list<int>& perm);
	void collapse();
	bool leaf() const { return next==0;}
	bool terminal() const { return next==0 || prune;}
	bool root() const { return prev==0;}
	int nchildren() const;
	int nterminals() const;
	void removechild(tnode *ch);
	void insertchild(tnode *ch);

	bool has_label() const { return lbl!=0;}
	void setlabel(const std::string &a);
	const char *label() const {return lbl;}
	
	
	
	void ladderize_sichtbare(int);
	void symladderize(int);
	void calcdepth_sichtbar();
	const tnode *unterster_sichtbarer() const;
	const tnode *oberster_sichtbarer() const;
	void sethpos(float,float);
	float sethpos_fixed(float x0,float w,float stdl);
	void sethpos_len(float x0,float sc);
	float optlen(std::vector<float>& x,int tief,float w);
	void getoptlenscale(float, float& smin, float& smax, const tree_desc&);
	float normlen_fixed(std::vector<float>& x,int tief,float w);
	float maxrund() const;
};



struct klammer
{
	nfstring text;
	std::string lb[2];//label-namen
	float sp;
	int style;

enum
{
	geschweift=0,
	eckig=1,
	gedreht=2
};

	klammer(){ sp=0; style=geschweift;}
	static float breite;
};




class label_list
{
	tnode **nd; //Knoten, nach labeln sortiert
	int nlabel;
	int nnodes;
	int autovar;
public:

	void random_label(char[]);

	label_list();
	~label_list();

	int total_nodes() const {return nnodes;}
	void init(int a);
	void einsortiere_label(tnode*);
	void kill_label(const char *s);
	const tnode *bfind_label(const char *s) const;

};



class tree : public tnode
{
	tree_desc dsc;
	std::list<klammer> kl;
	label_list lb;
	bool drty;
	std::list<std::string> definitionitems;//alles zwischen \begindef und \enddef

	void read_definitions(tgfParser&);
	void fill_labels(int); //Labels in sortierte Liste lb packen
	void user_labels(tnode*, label_list&);
	void auto_labels(tnode*, label_list&);
	
	
	void vrescale(float);
	void affine_transform(float m[4], float v[2]);
	void calcvpos();
	int calchpos();
	int calchpos_fixed();
	int calchpos_len();
	void calcpos();
	void calc_treewidth();
	void calc_mindimensions();
	
	bool reroot(tnode *);
	
	void remove_subtree(tnode *);


	void renderNode(const tnode*, gostream*) const;
	void renderRandklammern(gostream *) const;
	
	void writenodes(std::ostream& aus,const tnode* =0) const;
	void writenodesNEXUS(std::ostream& aus,const tnode* =0) const;

public:

	tree();
	~tree();
	
	void write(std::ostream&,const tnode* =0);
	void read(std::ifstream&);
	void render(gostream*) const;
	
	void writeNEXUS(std::ostream&);
		
	void edit();
	void show_dsc() const { dsc.show();};
	
	bool dirty() const {return drty;}

	const tree_desc get_desc() const {
		return dsc;
	}
};


float min_node_width(const tnode *p, const std::string& colsep, const tree_desc& dsc);




#endif //__TGF_TREE_H__
