
#include "tgf-treedesc.h"
#include "tgf-utils.h"
#include "tgf-parser.h"
#include "tgf-error.h"
#include "tgf-fonttable.h"

using namespace std;

tree_desc::tree_desc()
{
	uselength=false;
	shownumbers=false;
	roundness=0;
	//Stile vordefinieren
	//Bis auf default sind diese an feste Positionen gebunden
	// default wird immer genommen, wenn Stil undefiniert
	// Hier ist z.B auch der u2-Stil noch default
	setstyle("default",fontstyle("plain",12.0));
	setstyle("rule",fontstyle("bold",12.0));
	setstyle("ndl",fontstyle("bold",10.0));//node label
		
	extrasep=helv_stringwidth(nfstring("x","r"));
	picheight=200*millimeter;
	picwidth=150*millimeter;
	variablelength=false;
	thickness=0.33*millimeter;
	scale=1;
	bracketcolwidth=autolength;
	paper="a4";
	landscape=false;
	colsep=" ";
	treename="tree";
	rulepos=fPoint(4*millimeter,10*millimeter);
	tgfversion=0.9;//also total veraltet
	
	
}

tree_desc::tree_desc(const tree_desc& b)
{

	*this=b;
	style.clear();
	list<namedfontstyle>::const_iterator fit=b.style.begin();
	while(fit!=b.style.end()){
		setstyle(fit->name,*fit);
		++fit;
	}

}



void tree_desc::show() const
{
	cout << "\\thickness{" << thickness/millimeter << "}\n";
//	float klammerrand;//breite des klammerrands.
	cout << "\\width{"<<picwidth/millimeter<<"}\n";
	cout << "\\height{"<<picheight/millimeter<<"}\n";
//	float scale;
	cout << "\\roundness{"<< roundness << "}\n";
//	float bracketcolwidth; //Platz fuer Klammern am rechten Rand
	cout << "\\margin{"<<Rand.left/millimeter<<"}{"<<Rand.top/millimeter<<"}{";
	cout << Rand.right/millimeter <<"}{"<< Rand.bottom/millimeter << "}\n";
	if(forceauto) cout << "\\autolength\n";
	else cout << "\\autolength not set\n";
	if(variablelength) cout << "\\variable\n";
	else cout << "\\variable not set\n";
	if(shownumbers) cout << "\\proof\n";
	else cout << "\\proof not set\n";
	cout << "\\paper{"<<paper<< "}\n";
	if(landscape) cout << "\\landscape\n";
	else cout << "\\landscape not set (i.e. portrait)\n";
	cout << "\\separator{"<<colsep<<"}\n";
	cout << "\\rulepos{"<<rulepos.x/millimeter<< "}{"<<rulepos.y/millimeter<<"}\n";

	
	for(list<namedfontstyle>::const_iterator fit=style.begin();fit!=style.end();++fit){
		shownamedstyle(cout,*fit);
		cout << endl;
		
	}
	

}

void tree_desc::shownamedstyle(ostream& aus, const namedfontstyle& f)
{
	aus << "\\style{"<<f.name<<"}{";
	if(f.face==fs_italic)
		aus << "italic";
	else if(f.face==fs_bold)
		aus << "bold";
	else aus << "plain";
	aus << "}{" << f.fontsize<< "}";

}


void tree_desc::setstyle(const string& ident, const fontstyle& fs)
{
	list<namedfontstyle>::iterator fit=style.begin();
	while(fit!=style.end()){
		if(fit->name==ident){
			*fit=namedfontstyle(ident,fs);
			return;
		}
		++fit;
	}
	style.push_back(namedfontstyle(ident,fs));

}


fontstyle tree_desc::getstyle(const string& ident) const
{
	
	list<namedfontstyle>::const_iterator fit=style.begin();
	while(fit!=style.end()){
		if(fit->name==ident){
			return *fit;
		}
		++fit;
	}
	//not found: use default
	for(fit=style.begin();fit!=style.end();++fit){
		if(fit->name=="default"){
			return *fit;
		}
	}
	//still not found: may not happen
	throw tgfError::error("tree_desc::getstyle damaged");
}

bool tree_desc::styleexists(const std::string& ident) const
{
	list<namedfontstyle>::const_iterator fit=style.begin();
	while(fit!=style.end()){
		if(fit->name==ident){
			return true;
		}
		++fit;
	}
	return false;
}


float tree_desc::getfontsize(const nfstring& fs) const
{
	if(fs=="") return static_cast<float>(0);
	fontstyle fst=getstyle(fs.stylename);
	return fst.fontsize;

}

float tree_desc::helv_stringwidth(const std::string& s, const fontstyle& fst) const
{
	float sz=0;
	for(unsigned int i=0; i<s.length(); i++){
		float x;
		int c=s[i];
		if(c<32 || c>126){//unbekannter Buchstabe
			x=20.0;
		} else {
			c-=32;
			if(fst.face==fs_bold) c+=95;
			else if(fst.face==fs_italic) c+=190;
			x=helv24[c];
		}
		sz+=x;
	}
	float ll=sz*(fst.fontsize)/24.0;
	return ll;

}


float tree_desc::helv_stringwidth(const nfstring& s) const 
{
	fontstyle fst=getstyle(s.stylename);
	return helv_stringwidth(s,fst);
}



