/*
	tgf-utils.h, part of
	
	treegraph
	
	Tree formatting program
	
	Generates vector graphics (SVG,EPS) from .tgf-tree description files.

	Copyright (c) 2003-06 by Joern Mueller


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program (GPL.html); if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	(also: http://www.gnu.org)
*/







#ifndef __TGF_UTILS_H__
#define __TGF_UTILS_H__

#include <string>
#include <cstdio>

enum fontface {
	fs_plain,
	fs_bold,
	fs_italic
};

struct fontstyle
{
	fontface face;
	float fontsize;
	
	fontstyle(float f=10, fontface fc=fs_plain) : face(fc), fontsize(f) {}	
	fontstyle(const std::string& fn, float f) : fontsize(f) {
		if(fn=="bold"){
			face=fs_bold;
		} else if(fn=="italic"){
			face=fs_italic;
		} else {
			face=fs_plain;
		}
	}
};


/*
class fstring : public std::string, public fontstyle
{
public:
	fstring(const std::string& s="") : std::string(s), fontstyle() {}
	fstring(const std::string& s, fontstyle f) : std::string(s), fontstyle(f) {}
//	fstring(const fstring& f) : std::string(f), fontstyle(f) {}
};

*/

class namedfontstyle : public fontstyle
{
public:
	std::string name;
	namedfontstyle(const std::string& s="default") :  fontstyle(), name(s) {}
	namedfontstyle(const std::string& s, const fontstyle& f) : fontstyle(f), name(s) {}
//	namedfontstyle(const namedfontstyle& f) : name(f.name), fontstyle(f) {}
	
};


//
// nfstring: Ein string und ein Style-name. Diesem ist ein namedfontstyle zugeordnet
//
class nfstring : public std::string
{
public:
	std::string stylename;
	nfstring(const std::string& s="", const std::string& fsn="default") : std::string(s), stylename(fsn) {}
//	fstring(const fstring& f) : std::string(f), fontstyle(f) {}
};




class fPoint {
public:
	float x,y;

	fPoint(float xx=0, float yy=0) : x(xx), y(yy) {}
};

class fRect {
public:
	float left,top,right, bottom; //top>=bottom

	fRect(float l=0, float t=0, float r=0, float b=0) : left(l), top(t), right(r),bottom(b){}
	fRect(const fPoint& a, const fPoint& b){
		left=a.x;
		top=a.y;
		right=b.x;
		bottom=b.y;
	}

	float width() const { return right-left;}
	float height() const { return top-bottom;}
};




const float autolength= -1.0;

#endif /* __TGF_UTILS_H__ */

